#include <iostream>
#include <string>
using namespace std;

#include "console.h"
#include "simpio.h"
#include "queue.h"
#include "josephus-graphics.h"

static void determineSurvivors(JosephusWindow& window,
                               int numPeople, int numToSpare,
                               int& lastKilled, int& nextToLastKilled) {
    Queue<int> circle;
    for (int n = 1; n <= numPeople; n++) {
        circle.enqueue(n);
    }

    while (!circle.isEmpty()) {
        for (int numSpared = 0; numSpared < numToSpare; numSpared++) {
            int next = circle.dequeue();
            window.spare(next);
            circle.enqueue(next);
            window.resurrect(next);
        }

        nextToLastKilled = lastKilled;
        lastKilled = circle.dequeue();
        window.execute(lastKilled);
    }
}

static int promptForInteger(string prompt, int low, int high) {
    while (true) {
        int response = getInteger(prompt);
        if (response >= low && response <= high) return response;
        cout << "We need a number between " << low << " and " << high << ", inclusive.  Try again." << endl;
    }
}

int main() {
    setConsoleFont("Courier-BOLD-24");
    JosephusWindow window;
    cout << "Welcome to the Josephus survivors demonstration." << endl;
    do {
        window.clear();
        int numPeople = promptForInteger("Enter the number of people in the circle: ", 10, 70);
        window.setNumPeople(numPeople);
        int numToSpare = promptForInteger("Enter the number of people spared per iteration: ", 1, 2 * numPeople);
        int lastKilled = 0, nextToLastKilled = 0;
        determineSurvivors(window, numPeople, numToSpare, lastKilled, nextToLastKilled);
        cout << "Last two people executed: " << lastKilled << " and " << nextToLastKilled << endl;
        window.resurrect(lastKilled);
        window.resurrect(nextToLastKilled);
    } while (getLine("Again? ") != "no");
    return 0;
}
